#ifndef HASHMAP_H
#define HASHMAP_H

enum Rehash {LINEAR, QUADRATIC, DOUBLE};
enum SlotState {EMPTY, USED, DELETED};

template <class K>
int DefaultDoubleHashFn(const K & key) {
	return 1;
} 


template <class K, class V>
class HashEntry {

public:
	K key;
	V value;
	SlotState slotState;

	HashEntry() {
		slotState = EMPTY;
	}
};


template <class K, class V>
class HashMap {

private:
	HashEntry<K, V> * table;
	int tableSize;
	int numItems;
	int (*hashFunc)(const K &);
	Rehash rehashProbe;
	int (*doubleHashFunc) (const K &);

public:

	HashMap(int size, int (*hashFn)(const K &), Rehash probe = LINEAR, int (*doubleHashFn)(const K &) = DefaultDoubleHashFn );
	~HashMap();

	bool contains(K key);
	V get(K key); 
	void put(K key, V value);
	void remove(K key);
	int size() const;
	int getHashTableSize() const;
	void clear();
	void display();
	double loadFactor() const;
};

#endif