#include <iostream>
#include <string>
#include <vector>
#include <algorithm>
#include <cstdlib>
#include <ctime>
#include "HashMap.cpp"
using namespace std;

void printMenu(string & response) {
	
	cout << "Test Open-Address Hash Table" << endl<< endl;
	cout << "(p)ut\n";
	cout << "co(n)tains\n";
	cout << "(g)et\n";
	cout << "(r)emove\n";
	cout << "si(z)e\n";
	cout << "(h)ash table size\n";
	cout << "(c)lear\n";
	cout << "(d)isplay\n";
	cout << "(q)uit\n";
	cout << "\nWhat would you like to do? ";
	cin >> response;

} // end printMenu

int myHashFn(const int & key) {
	return key;
}

int main() {
	HashMap<int, string> myHashTable(10, myHashFn);
//	HashMap<int, string> myHashTable(10, myHashFn, QUADRATIC);
	string command;
	int key;
	string value;
	unsigned startTime, endTime;
	int count, numberOfElements;
	double loadFactor;

	while (true) {
		printMenu(command);
		if (command == "z" || command == "Z") {
			cout << "Number of items " << myHashTable.size() << endl;
		} else if (command == "p" || command == "p") {
			cout << "Enter the key to insert: ";
			cin >> key;
			cout << "Enter the value to insert: ";
			cin >> value;
			myHashTable.put(key, value);
		} else if (command == "n" || command == "N") {
			cout << "Enter the key to check if hash table contains: ";
			cin >> key;
			if (myHashTable.contains(key)) {
				cout << "myHashTable.contains(" << key << ") is true" << endl;
			} else {
				cout << "myHashTable.contains(" << key << ") is false" << endl;
			} // end if
		} else if (command == "g" || command == "G") {
			cout << "Enter the key to get: ";
			cin >> key;
			if (myHashTable.contains(key)) {
				cout << "myHashTable.get(" << key << ") returned: " << myHashTable.get(key) << endl;
			} else {
				cout << "myHashTable DOES NOT contain key (" << key << ")" << endl;
			} // end if
		} else if (command == "r" || command == "R") {
			cout << "Enter the value to remove: ";
			cin >> key;
			if (myHashTable.contains(key)) {
				myHashTable.remove(key);
			} else {
				cout << "myHashTable DOES NOT contain key (" << key << ")" << endl;
			} // end if
		} else if (command == "c" || command == "C") {
			myHashTable.clear();
			cout << "The hash table was cleared and now has " << myHashTable.size() << " elements." << endl;
		} else if (command == "h" || command == "H") {
			cout << "The hash table size is " << myHashTable.getHashTableSize() << " elements." << endl;
		} else if (command == "d" || command == "D") {
			cout << "The hash table with load factor (" << myHashTable.loadFactor() << ") is: " << endl;
			myHashTable.display();
		} else if (command == "q" || command == "Q") {
			break;
		} // end if

		cout << "\n===================================================" << endl;
	} // end while

	myHashTable.clear();
	cout << "\n=================================================\n\n";

/* This HashMap implementation uses open-address hashing.  The main.cpp program 
adds the 10,000 even values (i.e., 0, 2, 4, 6, 8, ..., 19996, 19998) to a HashTable of 
size 50,000 (i.e., load factor of 0.2) using linear probing for rehashing.  It times the 
searching for target values from 0, 1, 2, 3, 4, ..., 19998, 19999 so half of the searches 
are successful and half are unsuccessful.  How long does it take to search for target values 
from 0, 1, 2, 3, 4, ..., 19998, 19999 in the HashTable with load factor of 0.2?
*/
	numberOfElements = 100000;
	for (loadFactor = 0.2; loadFactor < 0.95; loadFactor = loadFactor + 0.1) {
		HashMap<int, string> myHashTable(static_cast<int>(numberOfElements/loadFactor), myHashFn, LINEAR);
//		HashMap<int, string> myHashTable(static_cast<int>(numberOfElements/loadFactor), myHashFn, QUADRATIC);
		
		// store even values 0, 2, 4, 6, 8, ..., 199996, 199998 to a HashTable
		for (count = 0; count < numberOfElements; count++) {
			myHashTable.put(count*2, "value");
		} // end for

		// search for target values from 0, 1, 2, 3, 4, ..., 199998, 199999
		startTime = static_cast<unsigned>(time(0));
		for (count = 0; count < numberOfElements*2; count++) {
			myHashTable.contains(count);
		} // end for
		endTime = static_cast<unsigned>(time(0));
		cout << "Hash Table size = " << myHashTable.getHashTableSize() << "  with 'load factor' = " << loadFactor 
			<< ", but actual load factor = " << myHashTable.loadFactor() <<	endl;
		cout << "Search time of " << numberOfElements*2 << " elements (half successful and half unsuccessful) took " 
			<< endTime - startTime << " seconds." << endl;	
		cout << "\n====================================================================\n";
	} // end for

	char dummy[50];
	cin.getline(dummy, 50);
	cout << "Hit any key to quit: ";
	cin.get();	
} // end main