#include "LinkedHashTable.h"
#include <cassert>

template <class K, class V>
HashMap<K, V>::HashMap(int size, int (*hashFn)(const K &)) {

	tableSize = size;
	numItems = 0;
	hashFunc = hashFn;
	table = new LinkedHashEntry<K, V>* [tableSize];

	for (int i = 0; i < tableSize; i++) {
		table[i] = NULL;
	} // end for
} // end HashMap constructor


template <class K, class V>
V HashMap<K, V>::get(K key) {

	int hash = (abs(hashFunc(key)) % tableSize);

	if (table[hash] == NULL) {
		assert(table[hash] != NULL);
	} else {
		LinkedHashEntry<K, V> *entry = table[hash];
		while (entry != NULL && entry->key != key) {
			entry = entry->next;
		} // end while
		if (entry == NULL) {
			assert(table[hash] != NULL);
		} else {
			return entry->value;
		} // end if
	} // end if
	return NULL; // should never get here

}


template <class K, class V>
bool HashMap<K, V>::contains(K key) {

	int hash = (abs(hashFunc(key)) % tableSize);

	if (table[hash] == NULL) {
		return false;
	} else {
		LinkedHashEntry<K, V> *entry = table[hash];
		while (entry != NULL && entry->key != key) {
			entry = entry->next;
		} // end while
		if (entry == NULL) {
			return false;
		} else {
			return true;
		} // end if
	} // end if
}



template <class K, class V>
void HashMap<K, V>::put(K key, V value) {

	int hash = (abs(hashFunc(key)) % tableSize);

	if (table[hash] == NULL) {
		table[hash] = new LinkedHashEntry<K, V>(key, value);
		numItems++;
	} else {
		LinkedHashEntry<K, V> *entry = table[hash];
		while (entry->next != NULL && entry->key != key) {
			entry = entry->next;
		} // end while

		if (entry->key == key) {
			entry->value = value;
		} else {
			entry->next = new LinkedHashEntry<K, V>(key, value);
			numItems++;
		} // end if

	} // end if
} // end put



template <class K, class V>
void HashMap<K, V>::remove(K key) {

	int hash = (abs(hashFunc(key)) % tableSize);

	if (table[hash] != NULL) {

		LinkedHashEntry<K, V> *prevEntry = NULL;
		LinkedHashEntry<K, V> *entry = table[hash];

		while (entry->next != NULL && entry->key != key) {
			prevEntry = entry;
			entry = entry->next;
		} // end while

		if (entry->key == key) {

			if (prevEntry == NULL) {
				LinkedHashEntry<K, V> *nextEntry = entry->next;
				delete entry;
				table[hash] = nextEntry;
			} else {
				LinkedHashEntry<K, V> *nextEntry = entry->next;
				delete entry;
				prevEntry->next = nextEntry;
			} // end if
			numItems--;

		} // end if

	} // end if

}  // end remove

template <class K, class V>
int HashMap<K, V>::size() const {
	return numItems;
} // end size


template <class K, class V>
double HashMap<K, V>::loadFactor() const {
	return double(numItems)/tableSize;
} // end loadFactor


template <class K, class V>
int HashMap<K, V>::getHashTableSize() const {
	return tableSize;
} // end size


template <class K, class V>
void HashMap<K, V>::clear() {
	LinkedHashEntry<K, V> * entryPtr;

	for (int i = 0; i < this->tableSize; i++) {

		while (table[i] != NULL) {
			entryPtr = table[i];
			table[i] = entryPtr->next;
			delete entryPtr;
		} // end while

	} // end for

	numItems = 0;

} // end clear



template <class K, class V>
HashMap<K, V>::~HashMap() {

	this->clear();

	delete[] table;
}  // end ~HashMap


template <class K, class V>
void HashMap<K, V>::display() {

	for (int i = 0; i < this->tableSize; i++) {
		cout << i <<": ";
		if (table[i] == NULL) {
			cout << endl;
		} else {
			LinkedHashEntry<K, V> *entry = table[i];
			while (entry != NULL) {
				cout << entry->key << "  ";
				entry = entry->next;
			} // end while
			cout << endl;
		} // end if

	} // end for

} // end display

