// Program to time the binary search algorithm and compare it to a
// closed-address hash table using unsorted linked lists at each 
// hash table slot.
#include <iostream>
#include <cstdlib>
#include <ctime>
using namespace std;
#include "LinkedHashTable.cpp"

// Prototypes
int binarySearch(int array[], int size, int value);

int myHashFn(const int & key) {
	return key;
}

int main() {
	int * evenList;
	int count, evenNumber, index, location;
	unsigned startTime, endTime;
	int numberOfElements;
	double loadFactor;

	cout << "Enter the number of even elements to fill array (20,000,000?): ";
	cin >> numberOfElements;
	evenList = new int [numberOfElements];

	// fill evenList with even values 0, 2, 4, ..., 39999996, 39999998
	evenNumber = 0;
	for (index = 0; index < numberOfElements; index++) {
		evenList[index] = evenNumber;
		evenNumber += 2;
	} // end for

	// search for target values from 0, 1, 2, 3, 4, ..., 39999998, 39999999
	startTime = static_cast<unsigned>(time(0));
	for (count = 0; count < numberOfElements*2; count++) {
		location = binarySearch(evenList, numberOfElements, count);
	} // end for
	endTime = static_cast<unsigned>(time(0));
	cout << "Binary Search time of " << numberOfElements*2 << " elements (half successful and half unsuccessful) took " 
		<< endTime - startTime << " seconds." << endl;	

	delete [] evenList;

	HashMap<int, string> * myHashTable = new HashMap<int, string>(numberOfElements, myHashFn);

	// store even values 0, 2, 4, 6, 8, ..., 39999996, 39999998 to a HashTable
	for (count = 0; count < numberOfElements; count++) {
		myHashTable->put(count*2, "value");
	} // end for

	// search for target values from 0, 1, 2, 3, 4, ..., 39999998, 39999999
	startTime = static_cast<unsigned>(time(0));
	for (count = 0; count < numberOfElements*2; count++) {
		myHashTable->contains(count);
	} // end for
	endTime = static_cast<unsigned>(time(0));
	cout << "Hash Table size = " << numberOfElements << "  with load factor = " << 1 << endl;
	cout << "Search time of " << numberOfElements*2 << " elements (half successful and half unsuccessful) took " 
		<< endTime - startTime << " seconds." << endl;	

	delete myHashTable;
	cout << "\n====================================================================\n";\

	numberOfElements = 20000000;
	for (loadFactor = 0.2; loadFactor < 30; loadFactor = loadFactor*2) {
		HashMap<int, string> myHashTable(static_cast<int>(numberOfElements/loadFactor), myHashFn);

		// store even values 0, 2, 4, 6, 8, ..., 199996, 199998 to a HashTable
		for (count = 0; count < numberOfElements; count++) {
			myHashTable.put(count*2, "value");
		} // end for

		// search for target values from 0, 1, 2, 3, 4, ..., 199998, 199999
		startTime = static_cast<unsigned>(time(0));
		for (count = 0; count < numberOfElements*2; count++) {
			myHashTable.contains(count);
		} // end for
		endTime = static_cast<unsigned>(time(0));
		cout << "Closed-address (linked list buckets) Hash Table size = " << myHashTable.getHashTableSize() << "  with load factor = " << loadFactor << endl;
		cout << "Search time of " << numberOfElements*2 << " elements (half successful and half unsuccessful) took " 
			<< endTime - startTime << " seconds." << endl;	
		cout << "\n====================================================================\n";
	} // end for


} // end main


//***************************************************************
// The binarySearch function performs a binary search on an     *
// integer array. array, which has a maximum of size elements,  *
// is searched for the number stored in value. If the number is *
// found, its array subscript is returned. Otherwise, -1 is     *
// returned indicating the value was not in the array.          *
//***************************************************************

int binarySearch(int array[], int size, int value)
{
	int first = 0,             // First array element
		last = size - 1,       // Last array element
		middle,                // Mid point of search
		position = -1;         // Position of search value
	bool found = false;        // Flag

	while (!found && first <= last)
	{
		middle = (first + last) / 2;     // Calculate mid point
		if (array[middle] == value)      // If value is found at mid
		{
			found = true;
			position = middle;
		}
		else if (array[middle] > value)  // If value is in lower half
			last = middle - 1;
		else
			first = middle + 1;           // If value is in upper half
	}
	return position;
}
